<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base\Admin;

use Infomaker\Everyware\Support\Str;

/**
 * Form
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\BaseAdmin
 * @since   Infomaker\Everyware\Base\Admin\Form 1.0.0
 */
class Form
{

    /**
     * @var string
     */
    private $fieldPrefix;

    /**
     * @var array
     */
    private $fields;


    public function __construct(string $fieldPrefix, array $fields = [])
    {
        $this->fieldPrefix = Str::slug($fieldPrefix);
        $this->fields = $fields;
    }

    public function getFormData(array $data = []): array
    {
        return $this->setupFields(array_replace_recursive($this->fields, $data));
    }

    private function setupFields(array $instance = []): array
    {
        $fieldsValues = [];

        foreach ($this->fields as $field => $value) {
            $fieldsValues[$field] = \is_array($value) ? $this->getFieldMultiValues($field,
                $instance[$field]) : $this->getFieldValues($field, $instance[$field]);
        }

        return $fieldsValues;
    }

    private function getFieldMultiValues(string $field, array $instance = []): array
    {
        $returnVal = [];
        foreach ($instance as $i => $value) {

            if ( ! \is_array($value)) {
                return $this->getFieldValues("{$field}[]", $instance);
            }

            foreach ((array)$value as $settingsName => $settingsValue) {
                $returnVal[$i][$settingsName] = $this->getFieldValues("{$field}[{$i}][{$settingsName}]",
                    $instance[$i][$settingsName]);
            }
        }

        return $returnVal;
    }

    /**
     * @param string $field
     * @param mixed  $value
     *
     * @return array
     */
    private function getFieldValues(string $field, $value): array
    {
        $fieldObj = new SettingsField($field, $value, $this->fieldPrefix);

        return $fieldObj->toArray();
    }
}
